/* osgCompute - Copyright (C) 2008-2009 SVT Group
 *                                                                     
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 3 of
 * the License, or (at your option) any later version.
 *                                                                     
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesse General Public License for more details.
 *
 * The full license is in LICENSE file included with this distribution.
*/

void main (void)
{
  vec4 result;

  // Note: That's defined in the PointSprite wrapper
  // (GLSL 120 will support gl_PointCoord?)
  vec2 tex_coord = gl_TexCoord[0].xy;
  tex_coord.y = 1.0-tex_coord.y;

  /////////////////////////
  // compute alpha value //
  /////////////////////////
  float d = 2.0*distance(tex_coord.xy, vec2(0.5, 0.5));
  result.a = step(d, 1.0);
  
  // Phong
  // -----
  vec3 eye_vector = normalize(vec3(0.0, 0.0, 1.0));
  vec3 light_vector = normalize(vec3(2.0, 2.0, 1.0));
  vec3 surface_normal = normalize(vec3(2.0*
    (tex_coord.xy-vec2(0.5, 0.5)), sqrt(1.0-d)));
  vec3 half_vector = normalize(eye_vector+light_vector);
  
  // Calculate specular component
  float specular = dot(surface_normal, half_vector);
  float diffuse  = dot(surface_normal, light_vector);

  // Use the lit function to compute lighting vector from diffuse and
  // specular values (does not work with GLSL at the moment)
  // vec4 lighting = lit(diffuse, specular, 8.0);
  vec4 lighting = vec4(0.75, max(diffuse, 0.0), pow(max(specular, 0.0), 40.0), 0.0);

  result.rgb = lighting.x*vec3(0.2, 0.8, 0.2)+lighting.y*vec3(0.6, 0.6, 0.6)+
               lighting.z*vec3(0.25, 0.25, 0.25);


  gl_FragColor = result;
}
